@@ -18,6 +18,8 @@
 *************************************************************************************/
 #include <ma_odbc.h>
 
+extern const char* DefaultPluginLocation;
+
 struct st_madb_isolation MADB_IsolationLevel[] =
 {
   {SQL_TRANSACTION_REPEATABLE_READ, "REPEATABLE READ"},
@@ -140,7 +142,7 @@ SQLRETURN MADB_DbcSetAttr(MADB_Dbc *Dbc, SQLINTEGER Attribute, SQLPOINTER ValueP
   if (!Dbc)
   {
     /* Todo: check */
-    if (Attribute != SQL_ATTR_TRACE ||
+    if (Attribute != SQL_ATTR_TRACE &&
         Attribute != SQL_ATTR_TRACEFILE)
       return SQL_INVALID_HANDLE;
     return SQL_SUCCESS;
@@ -609,10 +611,9 @@ SQLRETURN MADB_DbcConnectDB(MADB_Dbc *Connection,
   }
   else
   {
-    const char *DefaultLocation= MADB_GetDefaultPluginsDir(Connection);
-    if (DefaultLocation != NULL)
+    if (DefaultPluginLocation != NULL)
     {
-      mysql_optionsv(Connection->mariadb, MYSQL_PLUGIN_DIR, DefaultLocation);
+      mysql_optionsv(Connection->mariadb, MYSQL_PLUGIN_DIR, DefaultPluginLocation);
     }
   }
 
@@ -776,6 +777,10 @@ SQLRETURN MADB_DbcConnectDB(MADB_Dbc *Connection,
     mysql_optionsv(Connection->mariadb, MYSQL_OPT_SSL_ENFORCE, (const char*)&ForceTls);
   }
 
+  if (!MADB_IS_EMPTY(Dsn->SslCrl))
+  {
+    mysql_optionsv(Connection->mariadb, MYSQL_OPT_SSL_CRL, Dsn->SslCrl);
+  }
   if (!MADB_IS_EMPTY(Dsn->SslCrlPath))
   {
     mysql_optionsv(Connection->mariadb, MYSQL_OPT_SSL_CRLPATH, Dsn->SslCrlPath);
@@ -795,6 +800,11 @@ SQLRETURN MADB_DbcConnectDB(MADB_Dbc *Connection,
     mysql_optionsv(Connection->mariadb, MARIADB_OPT_TLS_PEER_FP_LIST, (void*)Dsn->TlsPeerFpList);
   }
 
+  if (!MADB_IS_EMPTY(Dsn->TlsKeyPwd))
+  {
+    mysql_optionsv(Connection->mariadb, MARIADB_OPT_TLS_PASSPHRASE, (void*)Dsn->TlsKeyPwd);
+  }
+
   if (!mysql_real_connect(Connection->mariadb,
       Dsn->Socket ? "localhost" : Dsn->ServerName, Dsn->UserName, Dsn->Password,
         Dsn->Catalog && Dsn->Catalog[0] ? Dsn->Catalog : NULL, Dsn->Port, Dsn->Socket, client_flags))
@@ -1336,7 +1346,7 @@ SQLRETURN MADB_DbcGetInfo(MADB_Dbc *Dbc, SQLUSMALLINT InfoType, SQLPOINTER InfoV
     MADB_SET_NUM_VAL(SQLUSMALLINT, InfoValuePtr, SQL_GB_NO_RELATION, StringLengthPtr);
     break;
   case SQL_IDENTIFIER_CASE:
-    MADB_SET_NUM_VAL(SQLUINTEGER, InfoValuePtr, SQL_IC_MIXED, StringLengthPtr);
+    MADB_SET_NUM_VAL(SQLUSMALLINT, InfoValuePtr, SQL_IC_MIXED, StringLengthPtr);
     break;
   case SQL_IDENTIFIER_QUOTE_CHAR:
     SLen= (SQLSMALLINT)MADB_SetString(isWChar ? &Dbc->Charset : NULL, (void *)InfoValuePtr, BUFFER_CHAR_LEN(BufferLength, isWChar), 
