@@ -59,7 +59,7 @@ MYSQL_STMT* MADB_NewStmtHandle(MADB_Stmt *Stmt)
 BOOL QueryIsPossiblyMultistmt(MADB_QUERY *Query)
 {
   return Query->QueryType != MADB_QUERY_CREATE_PROC && Query->QueryType != MADB_QUERY_CREATE_FUNC &&
-         Query->QueryType != MADB_QUERY_CREATE_DEFINER;
+         Query->QueryType != MADB_QUERY_CREATE_DEFINER && Query->QueryType != MADB_NOT_ATOMIC_BLOCK;
 }
 
 
@@ -80,7 +80,7 @@ int SqlRtrim(char *StmtStr, int Length)
   return Length;
 }
 
-
+/* Function assumes that the query is multistatement. And, e.g. STMT_COUNT(Stmt->Query) > 1 */
 unsigned int GetMultiStatements(MADB_Stmt *Stmt, BOOL ExecDirect)
 {
   int          i= 0;
@@ -101,7 +101,7 @@ unsigned int GetMultiStatements(MADB_Stmt *Stmt, BOOL ExecDirect)
       CloseMultiStatements(Stmt);
 
       /* Last paranoid attempt make sure that we did not have a parsing error.
-         More to preserve "backward-compatimility" - we did this before, but before trying to
+         More to preserve "backward-compatibility" - we did this before, but before trying to
          prepare "multi-statement". */
       if (i == 0 && Stmt->Error.NativeError !=1295 /*ER_UNSUPPORTED_PS*/)
       {
@@ -521,7 +521,6 @@ size_t MADB_GetOctetLength(MYSQL_FIELD *Field, unsigned short MaxCharLen)
   case MYSQL_TYPE_SHORT:
     return 2;
   case MYSQL_TYPE_INT24:
-    return 3;
   case MYSQL_TYPE_LONG:
     return 4;
   case MYSQL_TYPE_LONGLONG:
